package minicp.engine.constraints;

import minicp.engine.core.AbstractConstraint;
import minicp.engine.core.IntVar;

import java.util.HashSet;
import java.util.Set;

/**
 * Modulo constraint between two variables
 */
public class Modulo extends AbstractConstraint {

    private final IntVar x, y;
    private final int mod;

    private int[] domX;
    private int[] domY;

    private long[] supportedX;
    private long magic = 0L;

    /**
     * Creates a constraint such
     * that {@code x = y % mod}
     *
     * @param x   the left member
     * @param y   the right memer
     * @param mod the modulo value on y
     */
    public Modulo(IntVar x, IntVar y, int mod) { // x = y mod v
        super(x.getSolver());
        this.x = x;
        this.y = y;
        this.mod = mod;
    }

    @Override
    public void post() {

        domX = new int[x.size()];
        domY = new int[y.size()];
        supportedX = new long[mod];

        x.removeAbove(mod - 1);
        x.removeBelow(-mod + 1);
        x.propagateOnDomainChange(this);
        y.propagateOnDomainChange(this);

        propagate();

    }

    @Override
    public void propagate() {
        magic++; // implicitely removes all values in supportedX

        int sy = y.fillArray(domY);
        for (int i = 0; i < sy; i++) {
            int val = domY[i];
            if (x.contains(val % mod)) {
                supportedX[val % mod] = magic;
            } else {
                y.remove(val);
            }
        }

        int sx = x.fillArray(domX);
        for (int i = 0; i < sx; i++) {
            int val = domX[i];
            if (supportedX[val] != magic) {
                x.remove(val);
            }
        }
    }
}
