package minicp.enigma;

import minicp.cp.Factory;
import minicp.engine.core.IntVar;
import minicp.engine.core.Solver;
import minicp.search.DFSearch;
import minicp.search.SearchStatistics;
import minicp.util.Procedure;

import java.util.Random;

import static minicp.cp.BranchingScheme.and;
import static minicp.cp.Factory.makeDfs;

import static minicp.cp.BranchingScheme.firstFail;
import static minicp.cp.Factory.makeDfs;

public class CommercialEnigmaBreaker {
    public static void main(String[] args) {
        Alphabet alphabet = new Alphabet("ABCDEFGHIJKLMNOPQRSTUVWXYZ");

        int[][] perm = new int[][]{
                {15, 14, 5, 22, 24, 13, 25, 12, 18, 9, 1, 23, 20, 16, 17, 21, 7, 2, 3, 11, 4, 8, 6, 19, 0, 10},
                {21, 17, 13, 6, 0, 16, 10, 20, 3, 9, 14, 19, 4, 7, 2, 8, 12, 5, 22, 23, 24, 18, 25, 11, 15, 1},
                {8, 2, 16, 25, 15, 7, 1, 4, 22, 9, 17, 11, 24, 12, 5, 18, 20, 13, 6, 19, 0, 3, 10, 23, 21, 14}
        };

        int[] reflector = {10, 18, 16, 6, 15, 19, 3, 12, 20, 23, 0, 14, 7, 22, 11, 4, 2, 24, 1, 5, 8, 25, 13, 9, 17, 21};
        int[] plug_board = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25};

        System.out.println("\nEXERCICE 1\n");

        singleRotorEmulate(alphabet, perm[0], 3);


        System.out.println("\n\nEXERCICE 2\n");

        String cipher0 = "ROOETWKCYDXDKDGOLUIYRVKFLPD";
        singleRotorBreak(alphabet, perm[0], cipher0, "H");


        System.out.println("\n\nEXERCICE 3\n");

        String cipher1 = "HHWBGGLRDUSPEBBINYGKSGEAMQOSMEEHTGJKNTGCQURVMNEUBCTWGMHBK";
        multipleRotorsBreak(alphabet, new int[][]{perm[0]}, cipher0, "H");
        System.out.println();
        multipleRotorsBreak(alphabet, new int[][]{perm[0], perm[1]}, cipher1, "TH");


        System.out.println("\n\nEXERCICE 4");

        String text = "TESTFORMYENIGMACONSTRAINTPROGRAMMINGMODEL";

        Random rand = new Random();
        int[] key = new int[perm.length];
        for (int i = 0; i < 5; ++i) {

            EnigmaEmulator machine = new EnigmaEmulator(alphabet,
                    new int[][]{Permutation.random(alphabet.size()), Permutation.random(alphabet.size()), Permutation.random(alphabet.size())},
                    plug_board,
                    Permutation.random_symmetric(alphabet.size() / 2, alphabet.size()));

            for (int k = 0; k < key.length; ++k) {
                key[k] = rand.nextInt(alphabet.size());
            }

            System.out.println();

            machine.set_key(key);
            String ciphertext = machine.encrypt(text);
            System.out.println("Enigma  ciphertext: " + ciphertext);
            System.out.print("CPmodel ciphertext: ");
            commercialEnigmaEncode(machine, text, key);

            System.out.println();

            machine.set_key(key);
            String plaintext = machine.encrypt(ciphertext);
            System.out.println("Enigma  plaintext: " + plaintext);
            System.out.print("CPmodel plaintext: ");
            commercialEnigmaEncode(machine, ciphertext, key);

        }

        System.out.println("\n\nEXERCICE 5\n");

        String cipher2 = "NVGSEZEUFBEVTWCLPZARJKLFKNPFMBZYCZSLCKQKOUUZIHFHVAFDHPZOLIQTFSLJROUNGDXYLONQBLWKXZGRAH";
        EnigmaEmulator machine = new EnigmaEmulator(alphabet, perm, plug_board, reflector);
        commercialEnigmaBreak(machine, cipher2, "WE");

    }

    static public void singleRotorEmulate(Alphabet alphabet, int[] permutation, int length) {

        Solver cp = Factory.makeSolver(false);
        int N = alphabet.size();

        /*************** TODO *******************/

        IntVar[] plaintext = null;
        IntVar[] ciphertext = null;
        IntVar key = null;


        /*****************************************/
        DFSearch dfs = makeDfs(cp, and(firstFail(plaintext), firstFail(key)));

        dfs.onSolution(() -> {
                    alphabet.print(plaintext);
                    System.out.print(" -");
                    System.out.print(key);
                    System.out.print("-> ");
                    alphabet.print(ciphertext);
                    System.out.println();
                }
        );
        SearchStatistics stats = dfs.solve(stat -> stat.numberOfSolutions() >= 100);
        System.out.println(stats);

    }

    static public void singleRotorBreak(Alphabet alphabet, int[] rotor, String code, String crib) {

        char[] code_char = code.toCharArray();
        char[] crib_char = crib.toCharArray();
        Solver cp = Factory.makeSolver(false);
        int N = alphabet.size();
        int K = code_char.length;

        /*************** TODO *******************/
        IntVar[] plaintext = null;
        IntVar key = null;


        // You have to define the branching.
        // They are two possible order,
        // either you start branching on the plaintext variable,
        // or you start branching on the key variable.
        DFSearch dfs = makeDfs(cp, and(firstFail(/*TODO*/), firstFail(/*TODO*/)));

        /*****************************************/

        dfs.onSolution(() -> {
                    alphabet.print(plaintext);
                    System.out.print(" - ");
                    System.out.print(key);
                    System.out.println(" -> " + code);
                }
        );
        SearchStatistics stats = dfs.solve();
        System.out.println(stats);

    }


    static public void multipleRotorsBreak(Alphabet alphabet, int[][] rotors, String code, String crib) {

        char[] code_char = code.toCharArray();
        char[] crib_char = crib.toCharArray();
        Solver cp = Factory.makeSolver(false);

        int N = alphabet.size();
        int K = code_char.length;
        int M = rotors.length; // number of rotors

        /*************** TODO *******************/
        IntVar[] key = null;
        IntVar[] plaintext = null;

        DFSearch dfs = makeDfs(cp, and(firstFail(/*TODO*/), firstFail(/*TODO*/)));
        /*****************************************/

        dfs.onSolution(() -> {
                    alphabet.print(plaintext);
                    System.out.print(" - ");
                    for (int i = 0; i < M; ++i) {
                        System.out.print(key[i]);
                    }
                    System.out.println(" -> " + code);
                }
        );
        SearchStatistics stats = dfs.solve();
        System.out.println(stats);

    }


    static public void commercialEnigmaEncode(EnigmaEmulator machine, String plaintext, int[] key) {
        Alphabet alphabet = machine.get_alphabet();
        char[] plain_char = plaintext.toCharArray();
        Solver cp = Factory.makeSolver(false);

        int[][] frotors = machine.get_forward_rotors();
        int[] reflector = machine.get_reflector();

        int N = alphabet.size();
        int M = frotors.length;
        int K = plain_char.length;

        /*************** TODO *******************/
        IntVar[] ciphertext = null;

        /*****************************************/

        DFSearch dfs = makeDfs(cp, firstFail(ciphertext));

        dfs.onSolution(() -> {
                    alphabet.print(ciphertext);
                    System.out.println();
                }
        );
        SearchStatistics stats = dfs.solve();
        System.out.println(stats);
    }


    static public void commercialEnigmaBreak(EnigmaEmulator machine, String code, String crib) {
        Alphabet alphabet = machine.get_alphabet();
        char[] code_char = code.toCharArray();
        char[] crib_char = crib.toCharArray();
        Solver cp = Factory.makeSolver(false);

        int[][] frotors = machine.get_forward_rotors();
        int[] reflector = machine.get_reflector();

        int N = alphabet.size();
        int M = frotors.length;
        int K = code_char.length;

        /*************** TODO *******************/
        IntVar[] plaintext = null;
        IntVar[] key = null;

        DFSearch dfs = makeDfs(cp, and(firstFail(/*TODO*/), firstFail(/*TODO*/)));
        /*****************************************/

        dfs.onSolution(() -> {
                    System.out.print(" - ");
                    for (int i = 0; i < M; ++i) {
                        System.out.print(key[i]);
                    }
                    System.out.print(" -> ");
                    alphabet.print(plaintext);
                    System.out.println();
                }
        );
        SearchStatistics stats = dfs.solve();
        System.out.println(stats);
    }
}