package minicp.enigma;

import minicp.cp.Factory;
import minicp.engine.core.IntVar;
import minicp.engine.core.Solver;
import minicp.search.DFSearch;
import minicp.search.SearchStatistics;

import static minicp.cp.BranchingScheme.and;
import static minicp.cp.BranchingScheme.firstFail;
import static minicp.cp.Factory.makeDfs;

public class MilitaryEnigmaBreaker {

    public static void main(String[] args) {
        Alphabet alphabet = new Alphabet("ACDEGHILMNORSTUW");
        int[][] perm = new int[][]{{5, 15, 7, 6, 3, 4, 13, 9, 12, 0, 11, 8, 2, 1, 10, 14},
                {13, 3, 9, 11, 14, 1, 6, 2, 8, 0, 4, 7, 12, 10, 5, 15},
                {6, 11, 12, 0, 13, 15, 8, 7, 2, 5, 1, 14, 3, 9, 10, 4}
        };

        int[] reflector = new int[]{9, 15, 13, 14, 10, 6, 5, 11, 12, 0, 4, 7, 8, 2, 3, 1};
        int[] plug_board = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17}; // This is not the real plug board
        int nplugs = 2;


        System.out.println("\nEXERCICE 1\n");
        String cipher2 = "SRICOGSISNUGNHHDANALOTWTOOWCARRRENIHMUCNLLALODRDNIHILWILSMODTRTCOTSWTWADNCUEUIWCM";

        EnigmaEmulator machine = new EnigmaEmulator(alphabet, perm, plug_board, reflector);

        militaryEnigmaBreak(machine, nplugs, cipher2, "CONGRATULATION");

    }

    static public void militaryEnigmaBreak(EnigmaEmulator machine, int nplugs, String ciphertext, String crib) {
        Alphabet alphabet = machine.get_alphabet();
        char[] code_char = ciphertext.toCharArray();
        char[] crib_char = crib.toCharArray();
        Solver cp = Factory.makeSolver(false);

        int[][] frotors = machine.get_forward_rotors();
        int[] reflector = machine.get_reflector();

        int N = alphabet.size();
        int M = frotors.length;
        int K = code_char.length;

        /*************** TODO *******************/
        IntVar[] key = null;
        IntVar[] plug_board = null;
        IntVar[] plaintext = null;


        DFSearch dfs = makeDfs(cp, null/*TODO*/);
        /*****************************************/

        dfs.onSolution(() -> {
                    System.out.print(" -");
                    for (int i = 0; i < N; ++i) {
                        System.out.print(plug_board[i]);
                    }
                    System.out.print(" / ");
                    for (int i = 0; i < M; ++i) {
                        System.out.print(key[i]);
                    }
                    System.out.print(" -> ");
                    alphabet.print(plaintext);
                    System.out.println();
                }
        );

        SearchStatistics stats = dfs.solve();
        System.out.println(stats);

    }
}

