package minicp.enigma;

import minicp.cp.Factory;
import minicp.engine.constraints.Element1D;
import minicp.engine.core.IntVar;
import minicp.engine.core.Solver;
import minicp.search.DFSearch;
import minicp.search.SearchStatistics;

import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;

import static minicp.cp.BranchingScheme.firstFail;
import static minicp.cp.Factory.makeDfs;

public class Warmup {
    public static void main(String[] args) {

        Alphabet A = new Alphabet("'!\"#&()+,-./0123456789:;<=>?ABCDEFGHIJKLMNOPQRSTUVWXYZ[]_abcdefghijklmnopqrstuvwxyz{}\n ");
        int[] permutation = new int[]{30, 86, 35, 11, 57, 47, 20, 66, 45, 81, 65, 80, 84, 2, 62, 23, 68, 44, 25, 37, 74, 76, 82, 40, 28, 18, 29, 8, 36, 72, 50, 9, 83, 7, 19, 22, 34, 38, 56, 5, 46, 32, 52, 6, 69, 33, 1, 42, 26, 24, 31, 16, 0, 73, 60, 14, 53, 43, 48, 77, 39, 41, 85, 75, 79, 13, 49, 61, 70, 54, 67, 12, 64, 58, 27, 17, 63, 59, 3, 4, 21, 55, 15, 78, 10, 71, 51};


        System.out.println("\n\nEXERCICE 1\n");

        Warmup.transposeEncodeChar(A, permutation);

        System.out.println("\n\nEXERCICE 2\n");

        Warmup.transposeEncodeText(A, "IAMAREALSPYNOW", permutation);

        System.out.println();

        System.out.println("\n\nEXERCICE 3\n");

        String cypher = Warmup.fileToString(System.getProperty("user.dir") + "/../part2_encrypted.html");
        Warmup.transposeDecodeText(A, cypher, permutation, System.getProperty("user.dir") + "/../part2.html");

        System.out.println();
    }


    static public void transposeEncodeChar(Alphabet alphabet, int[] permutation) {

        Solver cp = Factory.makeSolver(false);

        /*************** TODO *******************/
        IntVar plainchar = null;
        IntVar cipherchar = null;
        /*****************************************/
        DFSearch dfs = makeDfs(cp, firstFail(plainchar));

        dfs.onSolution(() -> {
                    alphabet.print(new IntVar[]{plainchar});
                    System.out.print(" -> ");
                    alphabet.print(new IntVar[]{cipherchar});
                    System.out.println();
                }
        );
        SearchStatistics stats = dfs.solve();

        System.out.println(stats);
    }

    static public void transposeEncodeText(Alphabet alphabet, String plaintext, int[] permutation) {

        Solver cp = Factory.makeSolver(false);

        char[] chars = plaintext.toCharArray();
        int N = alphabet.size();
        int K = chars.length;

        /*************** TODO *******************/
        IntVar[] ciphertext = null;

        /*****************************************/

        DFSearch dfs = makeDfs(cp, firstFail(ciphertext));
        dfs.onSolution(() -> {
                    System.out.print(plaintext + " -> ");
                    alphabet.print(ciphertext);
                    System.out.println();
                }
        );
        SearchStatistics stats = dfs.solve();
        System.out.println(stats);
    }

    static public void transposeDecodeText(Alphabet alphabet, String ciphertext, int[] permutation, String file_path) {

        Solver cp = Factory.makeSolver(false);

        char[] chars = ciphertext.toCharArray();
        int N = alphabet.size();
        int K = chars.length;

        /*************** TODO *******************/
        IntVar[] plaintext = null;


        /*****************************************/


        DFSearch dfs = makeDfs(cp, firstFail(plaintext));
        dfs.onSolution(() -> {
                    Warmup.stringToFile(alphabet.to_str(plaintext), file_path);
                    System.out.println("Decoded file saved at :" + file_path);
                }
        );
        SearchStatistics stats = dfs.solve();
        System.out.println(stats);

    }

    public static String fileToString(String path){
        String output = "";
        try
        {
            output = new String ( Files.readAllBytes(Paths.get(path)) );
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        return output;
    }

    public static void stringToFile(String text, String path){
        try{
            FileOutputStream outputStream = new FileOutputStream(path);
            byte[] strToBytes = text.getBytes();
            outputStream.write(strToBytes);
            outputStream.close();
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }

}
